@@ -28,6 +28,10 @@ import click
 @click.option('--optimize', type=int, default=2, help='Optimize constant terms')
 @click.option('--max_calls', type=int, default=-1, help='Maximum number of function calls')
 @click.option('--max_iters', type=int, default=-1, help='Maximum number of Minuit iterations')
+@click.option('--batch_mode/--no-batch', default=False, help='Batch mode when evaluating likelihood')
+@click.option('--int_bin_precision', type=float, default=-1., help='Integrate the PDF over the bins '
+                                                                   'instead of using the probability '
+                                                                   'density at the bin centre')
 @click.option('--parallel', type=int, default=0, help='Parallelize job across different nuisance'+\
                                                       'parameters using N workers.'+\
                                                       'Use -1 for N_CPU workers.')
@@ -159,6 +163,10 @@ def likelihood_scan(**kwargs):
 @click.option('--fix-multi/--no-fix-cache',  default=True, help='Fix MultiPdf level 2')
 @click.option('--max_calls', type=int, default=-1, help='Maximum number of function calls')
 @click.option('--max_iters', type=int, default=-1, help='Maximum number of Minuit iterations')
+@click.option('--batch_mode/--no-batch', default=False, help='Batch mode when evaluating likelihood')
+@click.option('--int_bin_precision', type=float, default=-1., help='Integrate the PDF over the bins '
+                                                                   'instead of using the probability '
+                                                                   'density at the bin centre')
 @click.option('--constrain/--no-constrain', 'constrain_nuis', default=True, help='Use constrained NLL')
 def cls_limit(**kwargs):
     """
@@ -179,7 +187,7 @@ def compile_macros(macros):
     """
     from quickstats.utils.root_utils import compile_macro
     if macros is None:
-        macros = ["RooTwoSidedCBShape", "AsymptoticCLsTool", "RooDataSetObjects"]
+        macros = ["RooTwoSidedCBShape", "AsymptoticCLsTool", "RooFitObjects"]
     else:
         macros = macros.split(',')
     for macro in macros:
