@@ -35,7 +35,8 @@ class AsymptoticCLs(AnalysisObject):
                  eps:float=1.0, strategy:int=0, print_level:int=-1, 
                  num_cpu:int=1, offset:bool=True, optimize:bool=True,
                  fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1,
-                 max_iters:int=-1, constrain_nuis:bool=True, **kwargs):
+                 max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
+                 int_bin_precision:float=-1., **kwargs):
         """
         args:
             precision: precision in mu that defines iterative cutoff
@@ -65,7 +66,9 @@ class AsymptoticCLs(AnalysisObject):
             'fix_multi': fix_multi,
             'max_calls': max_calls,
             'max_iters': max_iters,
-            'constrain_nuis': constrain_nuis
+            'constrain_nuis': constrain_nuis,
+            'batch_mode': batch_mode,
+            'int_bin_precision': int_bin_precision
         }
         
         super().__init__(**config)
