@@ -4,7 +4,9 @@
 # Email: chi.lung.cheng@cern.ch
 ##################################################################################################
 import os
+import sys
 import math
+import logging
 import fnmatch
 from typing import List, Optional, Union, Dict
 
@@ -13,6 +15,9 @@ import ROOT
 import quickstats
 from quickstats.components.numerics import is_integer
 from quickstats.utils.root_utils import load_macro
+from quickstats.utils.io import VerbosePrint
+
+_PRINT_ = VerbosePrint("INFO")
 
 class ExtendedModel(object):
     
@@ -34,7 +39,7 @@ class ExtendedModel(object):
                  data_name:str="combData", snapshot_name:Optional[str]=None,
                  binned_likelihood:bool=True, tag_as_measurement:str="pdf_",
                  fix_cache:bool=True, fix_multi:bool=True, interpolation_code:int=-1,
-                 load_extension:bool=True):
+                 load_extension:bool=True, verbosity:Union[int, str]="INFO"):
         self.fname = fname
         self.ws_name = ws_name
         self.mc_name = mc_name
@@ -45,6 +50,7 @@ class ExtendedModel(object):
         self.fix_cache = fix_cache
         self.fix_multi = fix_multi
         self.interpolation_code = interpolation_code
+        _PRINT_.verbosity = verbosity
         if load_extension:
             self.load_extension()
         self.initialize()
@@ -83,13 +89,13 @@ class ExtendedModel(object):
             if not hasattr(ROOT, 'RooTwoSidedCBShape'):
                 result = load_macro('RooTwoSidedCBShape')
                 if hasattr(ROOT, 'RooTwoSidedCBShape'):
-                    print('INFO: Loaded extension module "RooTwoSidedCBShape"')
+                    _PRINT_.info('INFO: Loaded extension module "RooTwoSidedCBShape"')
         except Exception as e:
             print(e)
             
     @staticmethod
-    def load_RooDataSet_dict():
-        load_macro('RooDataSetObjects')
+    def load_RooFitObjects():
+        load_macro('RooFitObjects')
         
     @staticmethod
     def modify_interp_codes(ws, interp_code, classes=None):
@@ -100,43 +106,43 @@ class ExtendedModel(object):
                 if (component.IsA() == cls.Class()):
                     component.setAllInterpCodes(interp_code)
                     class_name = cls.Class_Name().split('::')[-1]
-                    print('INFO: {} {} interpolation code set to {}'.format(component.GetName(),
+                    _PRINT_.info('INFO: {} {} interpolation code set to {}'.format(component.GetName(),
                                                                             class_name,
                                                                             interp_code))
         return None
 
     @staticmethod
     def activate_binned_likelihood(ws):
-        print('INFO: Activating binned likelihood evaluation')
+        _PRINT_.info('INFO: Activating binned likelihood evaluation')
         for component in ws.components():
             if (component.IsA() == ROOT.RooRealSumPdf.Class()):
                 component.setAttribute('BinnedLikelihood')
-                print('INFO: Activated binned likelihood attribute for {}'.format(component.GetName()))
+                _PRINT_.info('INFO: Activated binned likelihood attribute for {}'.format(component.GetName()))
         return None
                           
     @staticmethod
     def set_measurement(ws, condition):
-        print('INFO: Activating measurements to reduce memory consumption')
+        _PRINT_.info('INFO: Activating measurements to reduce memory consumption')
         for component in ws.components():
             name = component.GetName()
             if ((component.IsA() == ROOT.RooAddPdf.Class()) and condition(name)):
                 component.setAttribute('MAIN_MEASUREMENT')
-                print('INFO: Activated main measurement attribute for {}'.format(name))
+                _PRINT_.info('INFO: Activated main measurement attribute for {}'.format(name))
         return None
     
     @staticmethod
     def deactivate_lv2_const_optimization(ws, condition):
-        print('INFO: Deactivating level 2 constant term optimization for specified pdfs')
+        _PRINT_.info('INFO: Deactivating level 2 constant term optimization for specified pdfs')
         for component in ws.components():
             name = component.GetName()
             if (component.InheritsFrom(ROOT.RooAbsPdf.Class()) and condition(name)):
                 component.setAttribute("NOCacheAndTrack")
-                print('INFO: Deactivated level 2 constant term optimization for {}'.format(name))
+                _PRINT_.info('INFO: Deactivated level 2 constant term optimization for {}'.format(name))
             
     def initialize(self):
         if not os.path.exists(self.fname):
             raise FileNotFoundError('workspace file {} does not exist'.format(self.fname))
-        print('INFO: Opening file "{}"'.format(self.fname))
+        _PRINT_.info('INFO: Opening file "{}"'.format(self.fname))
         file = ROOT.TFile(self.fname) 
         if (not file):
             raise RuntimeError("Something went wrong while loading the root file: {}".format(self.fname))
@@ -146,28 +152,28 @@ class ExtendedModel(object):
             if not ws_names:
                 raise RuntimeError("No workspaces found in the root file: {}".format(self.fname))
             if len(ws_names) > 1:
-                print("WARNING: Found multiple workspace instances from the root file: {}. Available workspaces"
+                _PRINT_.warning("WARNING: Found multiple workspace instances from the root file: {}. Available workspaces"
                       " are \"{}\". Will choose the first one by default".format(self.fname, ','.join(ws_names)))
             self.ws_name = ws_names[0]
 
         ws = file.Get(self.ws_name)
         if not ws:
             raise RuntimeError('failed to load workspace "{}"'.format(self.ws_name))
-        print('INFO: Loaded workspace "{}"'.format(self.ws_name))
+        _PRINT_.info('INFO: Loaded workspace "{}"'.format(self.ws_name))
         # load model config
         if self.mc_name is None:
             mc_names = [i.GetName() for i in ws.allGenericObjects() if 'ModelConfig' in i.ClassName()]
             if not mc_names:
                 raise RuntimeError("no ModelConfig object found in the workspace: {}".format(ws_name))
             if len(mc_names) > 1:
-                print("WARNING: Found multiple ModelConfig instances from the workspace: {}. "
+                _PRINT_.warning("WARNING: Found multiple ModelConfig instances from the workspace: {}. "
                       "Available ModelConfigs are \"{}\". "
                       "Will choose the first one by default".format(ws_name, ','.join(mc_names)))
             self.mc_name = mc_names[0]
         model_config = ws.obj(self.mc_name)
         if not model_config:
             raise RuntimeError('failed to load model config "{}"'.format(self.mc_name))
-        print('INFO: Loaded model config "{}"'.format(self.mc_name))
+        _PRINT_.info('INFO: Loaded model config "{}"'.format(self.mc_name))
             
         # modify interpolation code
         if self.interpolation_code != -1:
@@ -190,37 +196,37 @@ class ExtendedModel(object):
         pdf = model_config.GetPdf()
         if not pdf:
             raise RuntimeError('Failed to load pdf')
-        print('INFO: Loaded model pdf "{}" from model config'.format(pdf.GetName()))
+        _PRINT_.info('INFO: Loaded model pdf "{}" from model config'.format(pdf.GetName()))
              
         # load dataset
         data = ws.data(self.data_name)
         if not data:
             raise RuntimeError('Failed to load dataset')
-        print('INFO: Loaded dataset "{}" from workspace'.format(data.GetName()))
+        _PRINT_.info('INFO: Loaded dataset "{}" from workspace'.format(data.GetName()))
                 
         # load nuisance parameters
         nuisance_parameters = model_config.GetNuisanceParameters()
         if not nuisance_parameters:
             raise RuntimeError('Failed to load nuisance parameters')
-        print('INFO: Loaded nuisance parameters from model config')
+        _PRINT_.info('INFO: Loaded nuisance parameters from model config')
                 
         # Load global observables
         global_observables = model_config.GetGlobalObservables()
         if not global_observables:
             raise RuntimeError('Failed to load global observables')          
-        print('INFO: Loaded global observables from model config')                  
+        _PRINT_.info('INFO: Loaded global observables from model config')                  
     
         # Load POIs
         pois = model_config.GetParametersOfInterest()
         if not pois:
             raise RuntimeError('Failed to load parameters of interest')
-        print('INFO: Loaded parameters of interest from model config')
+        _PRINT_.info('INFO: Loaded parameters of interest from model config')
                                   
         # Load observables
         observables = model_config.GetObservables()
         if not observables:
             raise RuntimeError('Failed to load observables')     
-        print('INFO: Loaded observables from model config')
+        _PRINT_.info('INFO: Loaded observables from model config')
         
         self._file                = file
         self._workspace           = ws
@@ -235,7 +241,7 @@ class ExtendedModel(object):
         # Load snapshots
         if (self.snapshot_name) and self.workspace.getSnapshot(self.snapshot_name):
             self._workspace.loadSnapshot(self.snapshot_name)
-            print('INFO: Loaded snapshot "{}"'.format(self.snapshot_name))
+            _PRINT_.info('INFO: Loaded snapshot "{}"'.format(self.snapshot_name))
         return None
                 
     @staticmethod
@@ -264,7 +270,7 @@ class ExtendedModel(object):
         profiled_parameters = self._profile_parameters(self.workspace.allVars(), param_expr=param_expr,
                                                        param_str='parameter') 
         if not profiled_parameters:
-            print('Info: No parameters are profiled.')
+            _PRINT_.info('Info: No parameters are profiled.')
         return profiled_parameters 
     
     def fix_nuisance_parameters(self, param_expr=None):
@@ -284,7 +290,7 @@ class ExtendedModel(object):
         for name in param_dict:
             selected_params = [param for param in available_parameters if fnmatch.fnmatch(param, name)]
             if not selected_params:
-                print('WARNING: Parameter "{}" does not exist. No modification will be made.'.format(name))
+                _PRINT_.warning('WARNING: Parameter "{}" does not exist. No modification will be made.'.format(name))
             for param_name in selected_params:
                 ExtendedModel._set_parameter(source[param_name], param_dict[name], mode=mode, param_str=param_str)
                 set_parameters.append(source[param_name])
@@ -311,7 +317,7 @@ class ExtendedModel(object):
                 if (new_value < v_min) or (new_value > v_max):
                     new_value = (v_min + v_max)/2
                 param.setRange(v_min, v_max)
-                print('INFO: Set {} "{}" range to ({},{})'.format(param_str, name, v_min, v_max))
+                _PRINT_.info('INFO: Set {} "{}" range to ({},{})'.format(param_str, name, v_min, v_max))
             elif (v_min is not None):
                 if (new_value < v_min):
                     new_value = v_min
@@ -319,7 +325,7 @@ class ExtendedModel(object):
                 if (v_min == 0) and (old_value < 0):
                     new_value = abs(old_value)
                 param.setMin(v_min)
-                print('INFO: Set {} "{}" min value to ({},{})'.format(param_str, name, v_min))
+                _PRINT_.info('INFO: Set {} "{}" min value to ({},{})'.format(param_str, name, v_min))
             elif (v_max is not None):
                 if (new_value > v_max):
                     new_value = v_max
@@ -327,16 +333,16 @@ class ExtendedModel(object):
                 if (v_max == 0) and (old_value > 0):
                     new_value = -abs(old_value)                    
                 param.setMax(v_max)
-                print('INFO: Set {} "{}" max value to ({},{})'.format(param_str, name, v_max))
+                _PRINT_.info('INFO: Set {} "{}" max value to ({},{})'.format(param_str, name, v_max))
         if new_value != old_value:
             param.setVal(new_value)              
-            print('INFO: Set {} "{}" value to {}'.format(param_str, name, new_value))
+            _PRINT_.info('INFO: Set {} "{}" value to {}'.format(param_str, name, new_value))
         if mode=='fix':
             param.setConstant(1)
-            print('INFO: Fixed {} "{}" at value {}'.format(param_str, name, param.getVal()))
+            _PRINT_.info('INFO: Fixed {} "{}" at value {}'.format(param_str, name, param.getVal()))
         elif mode=='free':
             param.setConstant(0)
-            print('INFO: "{}" = [{}, {}]'.format(name, param.getMin(), param.getMax()))
+            _PRINT_.info('INFO: "{}" = [{}, {}]'.format(name, param.getMin(), param.getMax()))
         return None
 
     @staticmethod
@@ -467,14 +473,14 @@ class ExtendedModel(object):
                         target_np = np
                         break
             if not target_np:
-                print('WARNING: Could not find nuisance parameter for the constrain: {}'.format(pdf.GetName()))
+                _PRINT_.warning('WARNING: Could not find nuisance parameter for the constrain: {}'.format(pdf.GetName()))
                 continue
             for glob in globs:
                 if pdf.dependsOn(glob):
                     target_glob = glob
                     break
             if not target_glob:
-                print('WARNING: Could not find global observable for the constrain: {}'.format(pdf.GetName()))
+                _PRINT_.warning('WARNING: Could not find global observable for the constrain: {}'.format(pdf.GetName()))
                 continue
             nuis_list.append(target_np)
             glob_list.append(target_glob)
@@ -524,7 +530,7 @@ class ExtendedModel(object):
     
     def inspect_constrained_nuisance_parameter(self, nuis, constraints):
         nuis_name = nuis.GetName()
-        print('INFO: On nuisance parameter {}'.format(nuis_name))
+        _PRINT_.info('INFO: On nuisance parameter {}'.format(nuis_name))
         nuip_nom = 0.0
         prefit_variation = 1.0
         found_constraint = ROOT.kFALSE
@@ -554,19 +560,19 @@ class ExtendedModel(object):
                             if math.isclose(old_sigma_value, 1.0, abs_tol=0.001):
                                 old_sigma_value = 1.0
                             if not found_sigma:
-                                print('INFO: Sigma for pdf {} not found. Uisng 1.0.'.format(constr_name))
+                                _PRINT_.info('INFO: Sigma for pdf {} not found. Uisng 1.0.'.format(constr_name))
                             else:
-                                print('INFO: Uisng {} for sigma of pdf {}'.format(old_sigma_value, constr_name))
+                                _PRINT_.info('INFO: Uisng {} for sigma of pdf {}'.format(old_sigma_value, constr_name))
 
                             prefit_variation = old_sigma_value
                         elif constraint.IsA() == ROOT.RooPoisson.Class():
                             constraint_type = 'pois'
                             tau = glob_obs.getVal()
-                            print('INFO: Found tau {} of pdf'.format(constr_name))
+                            _PRINT_.info('INFO: Found tau {} of pdf'.format(constr_name))
                             prefit_variation = 1. / math.sqrt(tau)
-                            print('INFO: Prefit variation is {}'.format(prefit_variation))
+                            _PRINT_.info('INFO: Prefit variation is {}'.format(prefit_variation))
                             nuip_nom = 1.0
-                            print("INFO: Assume that {} is nominal value of the nuisance parameter".format(nuip_nom))
+                            _PRINT_.info("INFO: Assume that {} is nominal value of the nuisance parameter".format(nuip_nom))
         return prefit_variation, constraint_type, nuip_nom
         
     def set_initial_errors(self, source:Optional["ROOT.RooArgSet"]=None):
@@ -578,7 +584,7 @@ class ExtendedModel(object):
             nuis_name = nuis.GetName()
             prefit_variation, constraint_type, _ = self.inspect_constrained_nuisance_parameter(nuis, all_constraints)
             if constraint_type=='gaus':
-                print('INFO: Changing error of {} from {} to {}'.format(nuis_name, nuis.getError(), prefit_variation))
+                _PRINT_.info('INFO: Changing error of {} from {} to {}'.format(nuis_name, nuis.getError(), prefit_variation))
                 nuis.setError(prefit_variation)
                 nuis.removeRange()    
         return None
@@ -624,16 +630,38 @@ class ExtendedModel(object):
             raise ValueError('the argument "obs_and_weight" must be of RooArgSet type')
         return obs_and_weight, weight_var
     
+    @staticmethod
+    def get_object_map(object_dict:Dict, object_name:str):
+        ExtendedModel.load_RooFitObjects()
+        if object_name not in ["RooDataSet", "RooAbsPdf"]:
+            raise ValueError("unsupported object `{}`".format(object_name))
+        object_map = ROOT.std.map(f"string, {object_name}*")()
+        object_map.keepalive = list()
+        for c, d in object_dict.items():
+            object_map.keepalive.append(d)
+            object_map.insert(object_map.begin(), ROOT.std.pair(f"const string, {object_name}*")(c, d))
+        return object_map
+    
     @staticmethod
     def get_dataset_map(dataset_dict:Dict):
-        ExtendedModel.load_RooDataSet_dict()
+        ExtendedModel.load_RooFitObjects()
         dsmap = ROOT.std.map('string, RooDataSet*')()
         dsmap.keepalive = list()
         for c, d in dataset_dict.items():
             dsmap.keepalive.append(d)
-            dsmap.insert(dsmap.begin(), ROOT.std.pair("const string,RooDataSet*")(c, d))
+            dsmap.insert(dsmap.begin(), ROOT.std.pair("const string, RooDataSet*")(c, d))
         return dsmap
     
+    @staticmethod
+    def get_pdf_map(pdf_dict:Dict):
+        ExtendedModel.load_RooFitObjects()
+        pdfmap = ROOT.std.map('string, RooAbsPdf*')()
+        pdfmap.keepalive = list()
+        for c, d in pdf_dict.items():
+            pdfmap.keepalive.append(d)
+            pdfmap.insert(pdfmap.begin(), ROOT.std.pair("const string, RooAbsPdf*")(c, d))
+        return pdfmap    
+    
     def generate_asimov_from_pdf(self, name:str="asimovData", pdf:Optional["ROOT.RooAbsPdf"]=None,
                                  obs_and_weight:Optional[Union["ROOT.RooArgSet",str]]=None, 
                                  weight_var:Optional[Union["ROOT.RooRealVar",str]]=None,
@@ -667,7 +695,7 @@ class ExtendedModel(object):
             norm = pdf.getVal(obs)*target_obs.getBinWidth(i)
             n_events = norm*expected_events
             if n_events <= 0:
-                print("WARNING: Detected bin with zero expected events ({})! Please check"
+                _PRINT_.warning("WARNING: Detected bin with zero expected events ({})! Please check"
                       "your inputs. Obs = {}, bin = {}".format(n_events, target_obs.GetName(), i))
             elif (n_events > 0) and (n_events < 1e18):
                 asimov_data.add(self.observables, n_events)
@@ -706,10 +734,10 @@ class ExtendedModel(object):
         ws.saveSnapshot(tmp_glob_name, globs)
         ws.saveSnapshot(tmp_nuis_name, nuis)
         if not ws.getSnapshot(nom_glob_name):
-            print('INFO: Saving snapshot "{}"'.format(nom_glob_name))
+            _PRINT_.info('INFO: Saving snapshot "{}"'.format(nom_glob_name))
             ws.saveSnapshot(nom_glob_name, globs)
         if not ws.getSnapshot(nom_nuis_name):
-            print('INFO: Saving snapshot "{}"'.format(nom_nuis_name))
+            _PRINT_.info('INFO: Saving snapshot "{}"'.format(nom_nuis_name))
             ws.saveSnapshot(nom_nuis_name, nuis)
 
         if do_conditional and (poi_profile is not None):
@@ -791,7 +819,7 @@ class ExtendedModel(object):
             if not ws_names:
                 raise RuntimeError("No workspaces found in the root file: {}".format(fname))
             if len(ws_names) > 1:
-                print("WARNING: Found multiple workspace instances from the root file: {}. Available workspaces"
+                _PRINT_.warning("WARNING: Found multiple workspace instances from the root file: {}. Available workspaces"
                       " are \"{}\". Will choose the first one by default".format(fname, ','.join(ws_names)))
             ws_name = ws_names[0]
         ws = file.Get(ws_name)
@@ -803,7 +831,7 @@ class ExtendedModel(object):
             if not mc_names:
                 raise RuntimeError("no ModelConfig object found in the workspace: {}".format(ws_name))
             if len(mc_names) > 1:
-                print("WARNING: Found multiple ModelConfig instances from the workspace: {}. "
+                _PRINT_.warning("WARNING: Found multiple ModelConfig instances from the workspace: {}. "
                       "Available ModelConfigs are \"{}\". "
                       "Will choose the first one by default".format(ws_name, ','.join(mc_names)))
             mc_name = mc_names[0]     
