@@ -82,10 +82,11 @@ class NuisanceParameterPull(object):
                    precision:float=0.001, eps:float=1.0, strategy:int=0, 
                    num_cpu:int=1, offset:bool=True, optimize:int=2, eigen:bool=False,
                    fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
-                   max_iters:int=-1, outdir:str='output', cache=True, logfile_path=None, **kwargs):
+                   max_iters:int=-1, batch_mode:bool=False, int_bin_precision:float=-1.,
+                   outdir:str='output', cache=True, logfile_path=None, **kwargs):
         # create output directory if not exists
         if not os.path.exists(outdir):
-            os.makedirs(outdir)        
+            os.makedirs(outdir, exist_ok=True)        
         outname_root = os.path.join(outdir, nuis_name + '.root')
         outname_json = os.path.join(outdir, nuis_name + '.json')
         #checkpointing:
@@ -118,7 +119,7 @@ class NuisanceParameterPull(object):
         # profile pois
         if profile_param:
             print('INFO: Profiling POIs')
-            profile_pois = model.profile_parameters(profile)
+            profile_pois = model.profile_parameters(profile_param)
         
         buffer_time = time.time()
     
@@ -132,10 +133,14 @@ class NuisanceParameterPull(object):
         print("INFO: Making ExtendedMinimizer for unconditional fit")
         minimizer = ExtendedMinimizer("minimizer", model.pdf, model.data)
         print("INFO: Starting minimization")
-        nll_commands = [ROOT.RooFit.NumCPU(num_cpu, 3), 
+        nll_commands = [ROOT.RooFit.NumCPU(num_cpu, 3),
                         ROOT.RooFit.Constrain(model.nuisance_parameters),
                         ROOT.RooFit.GlobalObservables(model.global_observables), 
                         ROOT.RooFit.Offset(offset)]
+        if hasattr(ROOT.RooFit, "BatchMode"):
+            nll_commands.append(ROOT.RooFit.BatchMode(batch_mode))
+        if hasattr(ROOT.RooFit, "IntegrateBins"):
+            nll_commands.append(ROOT.RooFit.IntegrateBins(int_bin_precision))            
 
         minimize_options = {
             'minimizer_type'   : minimizer_type,
@@ -270,7 +275,8 @@ class NuisanceParameterPull(object):
                   minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, 
                   binned:bool=1, precision:float=0.001, eps:float=1.0, log_level:str='INFO', eigen:bool=False, 
                   strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, offset:bool=True, optimize:int=2,
-                  parameter:str="", max_calls:int=-1, max_iters:int=-1, parallel:int=0, cache:bool=True, 
+                  parameter:str="", max_calls:int=-1, max_iters:int=-1, batch_mode:bool=False, 
+                  int_bin_precision:float=-1.,parallel:int=0, cache:bool=True, 
                   exclude:str='', **kwargs):
         
         # configure default minimizer options
