@@ -1,5 +1,12 @@
+from typing import Union
+
 import numpy as np
 
+def pretty_float(val:float)->Union[int, float]:
+    if val.is_integer():
+        return int(val)
+    return val
+
 def is_integer(s:str):
     if not s:
         return False
